from .compressor import Compressor

__version__           = Compressor.__version__
__short_description__ = "Storer Tool"
__license__           = "MIT"
__author__            = "Alexander D. Kazakov"
__author_email__      = "alexander.d.kazakov@gmail.com"
__github_username__   = "AlexanderDKazakov"


# -*- coding: utf-8 -*-
from .storer import Storer

__all__ = (
    '__version__',
    'Storer'
)

__version__           = Storer.__version__
__short_description__ = "Minimalist storage class for any purpose."
__license__           = "MIT"
__author__            = "Alexander D. Kazakov"
__author_email__      = "alexander.d.kazakov@gmail.com"
__github_username__   = "AlexanderDKazakov"



import unittest
import os
import shutil
import time

from storer import Storer

PATH_DUMPS        = "./data/"
PATH_DUMPS_BACKUP = PATH_DUMPS + "backup/"
DUMP_NAME         = "test_case"
PKL               = ".pkl"
PBZ2              = ".pbz2"
VERBOSE           = False

class TestsStorer(unittest.TestCase):
    def test_creating_pkl(self):
        """
        Test paths of the created instance (PKL)
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, _test=True)
        s.put(1, name="one")
        s.dump()
        assert os.path.exists(os.path.expanduser(PATH_DUMPS) + DUMP_NAME + PKL)
        s._cleanup()

    def test_creating_pbz2(self):
        """
        Test paths of the created instance (PBZ2)
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, _test=True)
        s.put(1, name="one")
        s.dump()
        assert os.path.exists(os.path.expanduser(PATH_DUMPS) + DUMP_NAME + PBZ2)
        s._cleanup()
    
    def test_backup_dumb_pkl(self):
        """
        Test backup creating
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, _test=True)
        s.put(1, name="one")
        s.backup()
        assert os.path.exists(os.path.expanduser(PATH_DUMPS_BACKUP) + DUMP_NAME + PKL)
        s._cleanup()
    
    def test_backup_dumb_pbz2(self):
        """
        Test backup creating
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, _test=True)
        s.put(1, name="one")
        s.backup()
        assert os.path.exists(os.path.expanduser(PATH_DUMPS_BACKUP) + DUMP_NAME + PBZ2)
        s._cleanup()
    
    def test_get_item_pkl(self):
        """
        Test get item procedure
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, _test=True)
        s.put(1, name="one")
        s.put(2, name="two")
        three = s.get("three")
        assert three == False #  "Should be False!"
        s.put(3, name="three")
        s.dump()
        # here is new data in storer
        three = s.get("three")
        assert three == 3, f"got something different: [{three}]"  # "Should be 3!"
        s._cleanup()
    
    def test_get_item_bpz2(self):
        """
        Test get item procedure
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, _test=True)
        s.put(1, name="one")
        s.put(2, name="two")
        three = s.get("three")
        assert three == False #  "Should be False!"
        s.put(3, name="three")
        s.dump()
        # here is new data in storer
        three = s.get("three")
        assert three == 3, f"got something different: [{three}]"  # "Should be 3!"
        s._cleanup()
    
    def test_initialization_pkl(self):
        """
        Test initialization procedure
        Expected one backup file after dump procedure
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, _test=True)
        s.put(1, name="one")
        s.dump()
        s2 = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, _test=True)
        one = len(s2.backup_list)
        assert one == 1, f"Got something different: {one}"
        s._cleanup()
    
    def test_initialization_pbz2(self):
        """
        Test initialization procedure
        Expected one backup file after dump procedure
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, _test=True)
        s.put(1, name="one")
        s.dump()
        s2 = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, _test=True)
        one = len(s2.backup_list)
        assert one == 1, f"Got something different: {one}"
        s._cleanup()

    def test_separation(self):
        """
        Test separation procedure
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, separations=1, _test=True)
        s.put(1, name="one")    # 1
        s.put(2, name="two")    # 2
        s.put(3, name="three")  # 3
        s.put(4, name="four")   # 4
        s.dump()                
        s2 = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, _test=True)
        length = len(s2.backup_list)
        assert length == 4, f"Got something different: {length}"
        s._cleanup()
    
    def test_loading_separation(self):
        """
        Test separation procedure
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, separations=1, _test=True)
        s.put(1, name="one")    # 1
        s.put(2, name="two")    # 2
        s.put(3, name="three")  # 3
        s.put(4, name="four")   # 4
        s.dump()
        s2 = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, _test=True)
        one = s2.get(name="one")
        assert 1 == one, f"Got something different: {one}"
        two = s2.get(name="two")
        assert 2 == two, f"Got something different: {two}"
        three = s2.get(name="three")
        assert 3 == three, f"Got something different: {three}"
        four = s2.get(name="four")
        assert 4 == four, f"Got something different: {four}"
        s._cleanup()
    
    def test_loading_separation_pbz2(self):
        """
        Test separation procedure
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=True, separations=1, _test=True)
        s.put(1, name="one")    # _
        s.put(2, name="two")    # 0
        s.put(3, name="three")  # 1
        s.put(4, name="four")   # 2
        s.dump()                # 3
        s2 = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=True, _test=True)
        one = s2.get(name="one")
        assert 1 == one, f"Got something different: {one}"
        two = s2.get(name="two")
        assert 2 == two, f"Got something different: {two}"
        three = s2.get(name="three")
        assert 3 == three, f"Got something different: {three}"
        four = s2.get(name="four")
        assert 4 == four, f"Got something different: {four}"
        s._cleanup()

    def test_loop_loading(self):
        """
        Get method in loop:
        Checking for not overloading internal data dict
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, _test=True)
        cycles = 10
        for idx in range(cycles): 
            value = s.get(name=str(idx))
            if not value: s.put(what=idx, name=str(idx))
        s.dump()
        
        s2 = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, _test=True)
        for idx in range(cycles): 
            value = s.get(name=str(idx))
            assert value == idx, f"Got something different: {value} != {idx}"
        
        s._cleanup()

    def test_long_loop_loading(self):
        """
        Get method in loop:
        Checking for not overloading internal data dict
        """
        s = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, separations=10000, _test= True)
        cycles = 1000
        for idx in range(cycles): 
            value = s.get(name=str(idx))
            if not value: s.put(what=idx, name=str(idx))
        s.dump()

        s2 = Storer(dump_path=PATH_DUMPS, dump_name=DUMP_NAME, verbose=VERBOSE, compressed=False, separations=10000, _test= True)
        for idx in range(cycles): 
            value = s2.get(name=str(idx))
            assert value == idx, f"Got something different: {value} != {idx}"
        s._cleanup()


if __name__ == "__main__":
    unittest.main()



from dataclasses       import dataclass, field
from pathlib           import Path
from typing            import Any
import os, sys, shutil, signal, atexit
from storer.compressor import compressor

@dataclass
class Storer:
    __version__ = "1.0.8 [58]"
    internal_name:  str  = "[Storer]"
    dump_name:      str  = "noname"
    dump_path:      str  = Path(os.path.expanduser(os.path.dirname(__file__)))
    verbose:        bool = False
    exit_dump:      bool = True
    data:           dict = field(default_factory=dict)
    default_dir:    str  = "data"
    compressed:     bool = True
    separations:    int  = int(1e6)
    _backup_dir:    str  = "backup"
    backup_list:    list = field(default_factory=list)
    _put_counter:   int  = 0
    _dump_counter:  int  = 0
    _dump_name:     str  = None
    _extension:     str  = None

    def __post_init__(self):
        if self.verbose: print(f"[Storer v.{self.__version__ }] is initialized!")
        self._dump_name = self.dump_name
        if self.dump_path == Path(os.path.expanduser(os.path.dirname(__file__))) or self.dump_path == "." :
            self.dump_path = Path(os.path.expanduser(os.path.dirname(__file__))) / "data"
        else:
            self.dump_path = Path(os.path.expanduser(self.dump_path))

        if self.verbose: print(f"Dump folder: [{self.dump_path}]")
        os.makedirs(self.dump_path, exist_ok=True)
        self._extension = ".pbz2" if self.compressed else ".pkl"
        self._initialization()  # creating _backup_list
        self.compressor = compressor.Compressor(compressed = self.compressed,
                                                dump_path  = self.dump_path,
                                                dump_name  = self.dump_name)
        if self.exit_dump: atexit.register(self.dump)

    def _exit(self, signum, frame):
        if self.exit_dump: self.dump()
        sys.exit(0)

    def _cleanup(self) -> None:
        """
        Cleanup the dump_path directory fully: including all folders and files.
        Assuming the folder is used only for Storer purposes.
        """
        if self.verbose: print(f"Cleaning...[{self.dump_path}]")
        shutil.rmtree(self.dump_path)

    def _get_priv_dump_name(self) -> None:
        self.dump_name      = self._dump_name + "_" + str(self._dump_counter)
        self._dump_counter -= 1
        if self._dump_counter < 0: self._dump_counter = 0

    def _get_next_dump_name(self) -> None:
        self.dump_name      = self._dump_name + "_" + str(self._dump_counter)
        self._dump_counter += 1

    def _initialization(self) -> None:
        """
        Internal needs.
        """
        signal.signal(signal.SIGINT, self._exit)
        _backup_list = [p for p in self.dump_path.iterdir() if p.is_file() and str(p).endswith((".pkl", "gzip", "bz2", "lzma"))]
        for path_fname in _backup_list: self.backup_list.append(str(path_fname.name).split(self._extension)[0])

        if self.verbose:
            if len(self.backup_list):
                print(f"{self.internal_name} [BACKUPS] Found: ")
                for path_fname in self.backup_list: print(f"    --> {path_fname}")
            else: print(f"{self.internal_name} No data is available for loading...")

        if len(self.backup_list) == 0: self.backup_list.append(self.dump_name)
        self.backup_list.sort()

    def put(self, what=None, name: str = None) -> None:
        """
        Put an element to internal field of data
        """
        self.data[name]   = what
        self._put_counter+=1
        if self._put_counter >= self.separations:
            self.dump(_next_dump_name=True)
            self._get_next_dump_name()
            self._put_counter   = 0

    def get(self, name: str = None) -> Any:
        """
        Get an item from dump[s]
        """
        if name in self.data: return self.data[name]
        for dump_name in self.backup_list:
            data = self._load(dump_name=str(dump_name))
            if name in data: self.data = data; return data[name]
        return False

    def dump(self, backup:bool = False, _next_dump_name:bool = False) -> None:
        """
        Create a dump.

        Typical usage: dump()
        """
        if backup: dump_path = self.dump_path / self._backup_dir
        else:      dump_path = self.dump_path

        if self.data:
            if self.verbose: print(f"{self.internal_name} Path:{self.dump_path} Name:{self.dump_name} dumping...")
            self.compressor.dump(dump_path=dump_path, dump_name=self.dump_name, data=self.data)
            if backup or _next_dump_name: self.data = dict()

    def _load(self, dump_name:str = None) -> dict:
        """
        Internal needs.
        """
        if not dump_name: dump_name = self.dump_name
        data = self.compressor.load(dump_path=self.dump_path, dump_name=dump_name)
        return data

    def show(self, get_string = False) -> Any:
        """
        The show method: will show what currently is in the internal data
        """
        string = ""
        for name in self.data:
            string += "key: {0:10} | value:  {1:4}; ".format(name, str(self.data[name]))
        if get_string: return string
        else: print(string)

    def backup(self) -> None:
        """
        Backuping the current dump_name in separate folder [<dump_path> / backup ]
        """
        if self.verbose: print(f"Backup...")
        os.makedirs(self.dump_path / self._backup_dir, exist_ok=True)
        self.dump(backup=True)




import os
from dataclasses import dataclass
from pathlib     import Path
from enum        import Enum

import pickle
import bz2
import lzma
import gzip
import io
import time

class Algorithm(Enum):
    gzip = "gzip"
    bz2  = "bz2"
    lzma = "lzma"
    lz4   = ""

_file_algorithm_mapper = {
    Algorithm.gzip: [gzip.GzipFile, ".pgz"],
    Algorithm.bz2:  [bz2.BZ2File,   ".pbz2"],
    Algorithm.lzma: [lzma.LZMAFile, ".xz"],
}

@dataclass
class Compressor:
    __version__ = "0.0.3 [3]"
    compressed: bool = True
    algorithm : Algorithm = Algorithm.bz2
    dump_path : str        = Path(os.path.expanduser(os.path.dirname(__file__)))
    dump_name : str        = "noname"

    def _use(self, algorithm: Algorithm) -> None:
        self.algorithm = algorithm 

    def dump(self, dump_path:Path, dump_name:str, data:dict) -> None:
        if not self.compressed:
            path2file= dump_path / (dump_name + ".pkl")
            with open(path2file, "wb") as f: pickle.dump(data, f)
        else:
            path2file= dump_path / (dump_name + _file_algorithm_mapper[self.algorithm][1])
            with _file_algorithm_mapper[self.algorithm][0](path2file, "wb") as f: 
                pickle.dump(data, f)

    def load(self, dump_path:Path, dump_name:str) -> dict:
        if not self.compressed:
            path2file = dump_path / (dump_name + ".pkl") 
            if os.path.exists(path2file): 
                with open(path2file, "rb") as f: data = pickle.load(f)
            else: data=dict()
        else:
            path2file = dump_path / (dump_name + _file_algorithm_mapper[self.algorithm][1]) 
            if os.path.exists(path2file):
                with open(path2file, "rb") as f:
                        data = _file_algorithm_mapper[self.algorithm][0](filename=f, mode="rb")
                        data = pickle.load(data)
            else: data=dict()
        return data


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name         = 'storer',
    version      = '1.0.8',
    author       = 'Alexander D. Kazakov',
    author_email = 'alexander.d.kazakov@gmail.com',
    description  = 'Minimalist storage class for any purpose.',
    license      = 'MIT',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url          = 'https://github.com/AlexanderDKazakov/Storer',
    packages     =  setuptools.find_packages(),
    keywords     = ['store', 'pickle'],
    classifiers=[
      'Development Status :: 4 - Beta',
      'Intended Audience :: Developers',
      'Topic :: Software Development :: Build Tools',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.8',
      'Programming Language :: Python :: 3.9',
      ],
    python_requires='>=3.7',
    install_requires=[
          '',
      ],
)

